"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EggInstrumentation = exports.defaultConfig = void 0;
const api = __importStar(require("@opentelemetry/api"));
const instrumentation_1 = require("@opentelemetry/instrumentation");
const core_1 = require("@opentelemetry/core");
const types_1 = require("./types");
const AttributeNames_1 = require("./enums/AttributeNames");
const version_1 = require("./version");
const utils_1 = require("./utils");
exports.defaultConfig = {
    ignoreLayers: [
        (name) => {
            return [
                'middleware - meta',
                'middleware - siteFile',
                'middleware - notfound',
                'middleware - static',
                'middleware - overrideMethod',
                'middleware - session',
                'middleware - i18n',
                'middleware - bodyParser',
                'middleware - securities',
                'middleware - eggLoaderTrace',
            ].includes(name);
        },
    ],
};
class EggInstrumentation extends instrumentation_1.InstrumentationBase {
    constructor(config) {
        super('@opentelemetry/instrumentation-egg', version_1.VERSION, {
            ...exports.defaultConfig,
            ...config,
        });
    }
    init() {
        return new instrumentation_1.InstrumentationNodeModuleDefinition('egg-core', ['^4.20.0'], (moduleExports) => {
            if (moduleExports == null) {
                return moduleExports;
            }
            api.diag.debug('Patching Egg');
            const eggCoreProto = moduleExports.EggCore.prototype;
            if ((0, instrumentation_1.isWrapped)(eggCoreProto.use)) {
                this._unwrap(eggCoreProto, 'use');
            }
            this._wrap(eggCoreProto, 'use', this._getKoaUsePatch.bind(this));
            return moduleExports;
        }, (moduleExports) => {
            api.diag.debug('Unpatching Egg');
            if ((0, instrumentation_1.isWrapped)(moduleExports.EggCore.prototype.use)) {
                this._unwrap(moduleExports.EggCore.prototype, 'use');
            }
        });
    }
    _getKoaUsePatch(original) {
        const plugin = this;
        return function use(middlewareFunction) {
            let patchedFunction;
            if (middlewareFunction.router) {
                patchedFunction = plugin._patchRouterDispatch(middlewareFunction);
            }
            else {
                patchedFunction = plugin._patchLayer(middlewareFunction, false);
            }
            return original.apply(this, [patchedFunction]);
        };
    }
    _patchRouterDispatch(dispatchLayer) {
        var _a;
        api.diag.debug('Patching @koa/router dispatch');
        const router = dispatchLayer.router;
        const routesStack = (_a = router === null || router === void 0 ? void 0 : router.stack) !== null && _a !== void 0 ? _a : [];
        for (const pathLayer of routesStack) {
            const path = pathLayer.path;
            const pathStack = pathLayer.stack;
            for (let j = 0; j < pathStack.length; j++) {
                const routedMiddleware = pathStack[j];
                pathStack[j] = this._patchLayer(routedMiddleware, true, path);
            }
        }
        return dispatchLayer;
    }
    _patchLayer(middlewareLayer, isRouter, layerPath) {
        if (middlewareLayer[types_1.kLayerPatched] === true)
            return middlewareLayer;
        middlewareLayer[types_1.kLayerPatched] = true;
        api.diag.debug('patching Koa middleware layer');
        const metadata = (0, utils_1.getMiddlewareMetadata)(null, middlewareLayer, isRouter, layerPath);
        if ((0, utils_1.isLayerIgnored)(metadata.name, metadata.attributes[AttributeNames_1.AttributeNames.KOA_TYPE], this._config)) {
            api.diag.debug(`ignore Koa middleware layer ${metadata.name}`);
            return middlewareLayer;
        }
        return async (context, next) => {
            const parent = api.trace.getSpan(api.context.active());
            if (parent === undefined) {
                return middlewareLayer(context, next);
            }
            const metadata = (0, utils_1.getMiddlewareMetadata)(context, middlewareLayer, isRouter, layerPath);
            const span = this.tracer.startSpan(metadata.name, {
                attributes: metadata.attributes,
            });
            const rpcMetadata = (0, core_1.getRPCMetadata)(api.context.active());
            if (metadata.attributes[AttributeNames_1.AttributeNames.KOA_TYPE] === types_1.KoaLayerType.ROUTER &&
                (rpcMetadata === null || rpcMetadata === void 0 ? void 0 : rpcMetadata.type) === core_1.RPCType.HTTP) {
                rpcMetadata.span.updateName(`${context.method} ${context._matchedRoute}`);
            }
            let newContext = api.trace.setSpan(api.context.active(), span);
            if ((rpcMetadata === null || rpcMetadata === void 0 ? void 0 : rpcMetadata.type) === core_1.RPCType.HTTP) {
                newContext = (0, core_1.setRPCMetadata)(newContext, Object.assign(rpcMetadata, { route: context._matchedRoute }));
            }
            return api.context.with(newContext, async () => {
                try {
                    return await middlewareLayer(context, next);
                }
                catch (err) {
                    span.recordException(err);
                    throw err;
                }
                finally {
                    span.end();
                }
            });
        };
    }
}
exports.EggInstrumentation = EggInstrumentation;
EggInstrumentation.component = types_1.KoaComponentName;
//# sourceMappingURL=instrumentation.js.map